require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_merchant_onboarding/models/merchant_upload'
require_relative '../../mastercard_merchant_onboarding/models/merchant_download'
 

module MastercardMerchantOnboarding
 module Api

  class SingleMerchantUploadApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMerchantOnboarding::Tracker


    # Single Merchant Upload Service
    # This service is used by service providers to onboard merchant information one at a time to Masterpass through the project. Returns a MerchantDownload payload that will contain records of merchants that have, or have not, enabled 3-D Secure.
    # @param project_id Identifier of project created in the Merchant Portal.
    # @param merchant_upload Merchant upload request.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [MerchantDownload]
    def self.create(project_id, merchant_upload, api_config = nil)
		 path = "/masterpasspsp/v6/checkoutproject/{projectId}/file"
	 	 service_request = ServiceRequest.new
     service_request.path_params["projectId"] = project_id
	 
	   
	   service_request.body = merchant_upload
	 
	   service_request.content_type = "application/xml"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "POST",MerchantDownload)
    end

  end

 end
end



