require 'uri'
require 'mastercard_core_sdk'
require_relative '../../mastercard_merchant_onboarding/models/merchant_upload'
require_relative '../../mastercard_merchant_onboarding/models/validate_file_response'
 

module MastercardMerchantOnboarding
 module Api

  class SingleMerchantValidateApi
    include MastercardCoreSdk::Core, MastercardCoreSdk::Client, MastercardCoreSdk::Exceptions
	  include MastercardMerchantOnboarding::Tracker


    # Single Merchant Validate Service.
    # This service is used by service providers to validate merchant upload prior to actual onboarding submission. This process mimics the validation XML process via the user interface used within the merchant portal. Returns a ValidateFileResponse payload that will contain 3DS and/or non-3DS merchant status records.
    # @param project_id Identifier of project created in the Merchant Portal.
    # @param merchant_upload Merchant upload request.
    # @param api_config Optional ApiConfig object specifying configuration : consumer key, private key, host URL. 
    # @return [ValidateFileResponse]
    def self.create(project_id, merchant_upload, api_config = nil)
		 path = "/masterpasspsp/v6/checkoutproject/{projectId}/file/validation"
	 	 service_request = ServiceRequest.new
     service_request.path_params["projectId"] = project_id
	 
	   
	   service_request.body = merchant_upload
	 
	   service_request.content_type = "application/xml"
	   api_client = ApiClient.new(api_config)
     api_client.api_tracker = SdkApiTracker.new
     api_client.error_handler = ErrorHandler.new
	   return api_client.call(path, service_request, "POST",ValidateFileResponse)
    end

  end

 end
end



