require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
 

module MastercardMerchantOnboarding
  # This class contains merchant's acquirer details.
  class Acquirer
    include ROXML
    
    xml_name "Acquirer"
    
    # @!attribute action
    # @return [String] the merchant acquirer action. Valid values are C(Create), U(Update), and D (Delete).
    xml_accessor :action, :from =>"Action"
   
    # @!attribute id
    # @return [String] the merchant's acquirer id. For example, 453214.
    xml_accessor :id, :from =>"Id"
   
    # @!attribute name
    # @return [String] the merchant's acquirer name. For example, Test Acquirer. 3DS merchants must reduce their character limit to 250 to meet Cardinal Commerce’s API specifications.
    xml_accessor :name, :from =>"Name"
   
    # @!attribute assigned_merchant_id
    # @return [String] the acquirer assigned merchant id. For example, 435t6543.
    xml_accessor :assigned_merchant_id, :from =>"AssignedMerchantId"
   
    # @!attribute password
    # @return [String] the acquirer password for 3DS setup. For example, Visa. 3DS merchants must reduce their character limit to 8 to meet Cardinal Commerce’s API specifications.
    xml_accessor :password, :from =>"Password"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :action => :Action ,
        :id => :Id ,
        :name => :Name ,
        :assigned_merchant_id => :AssignedMerchantId ,
        :password => :Password ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:action)
        self.action = attributes[:action]
      end
      
      if attributes.has_key?(:id)
        self.id = attributes[:id]
      end
      
      if attributes.has_key?(:name)
        self.name = attributes[:name]
      end
      
      if attributes.has_key?(:assigned_merchant_id)
        self.assigned_merchant_id = attributes[:assigned_merchant_id]
      end
      
      if attributes.has_key?(:password)
        self.password = attributes[:password]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          action == o.action &&
          id == o.id &&
          name == o.name &&
          assigned_merchant_id == o.assigned_merchant_id &&
          password == o.password &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [action, id, name, assigned_merchant_id, password, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Acquirer" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :action => 'String',
        :id => 'String',
        :name => 'String',
        :assigned_merchant_id => 'String',
        :password => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
