require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
 

module MastercardMerchantOnboarding
  # This class contains merchant's address details.
  class Address
    include ROXML
    
    xml_name "Address"
    
    # @!attribute city
    # @return [String] the merchant's city. For example, New York.
    xml_accessor :city, :from =>"City"
   
    # @!attribute country
    # @return [String] the merchant's country. Standard 2-letter ISO 3166 country codes. For example, US.
    xml_accessor :country, :from =>"Country"
   
    # @!attribute country_subdivision
    # @return [String] the merchant’s country subdivision. Defined by ISO 3166-1 alpha-2 digit code. For example, US-VA is Virginia, US-OH is Ohio.
    xml_accessor :country_subdivision, :from =>"CountrySubdivision"
   
    # @!attribute line1
    # @return [String] the merchant’s address line 1. For example, 1 Main Street.
    xml_accessor :line1, :from =>"Line1"
   
    # @!attribute line2
    # @return [String] the merchant’s address line 2.
    xml_accessor :line2, :from =>"Line2"
   
    # @!attribute line3
    # @return [String] the merchant’s address line 3.
    xml_accessor :line3, :from =>"Line3"
   
    # @!attribute postal_code
    # @return [String] the merchant’s postal code. For example, 10001.
    xml_accessor :postal_code, :from =>"PostalCode"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :city => :City ,
        :country => :Country ,
        :country_subdivision => :CountrySubdivision ,
        :line1 => :Line1 ,
        :line2 => :Line2 ,
        :line3 => :Line3 ,
        :postal_code => :PostalCode ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:city)
        self.city = attributes[:city]
      end
      
      if attributes.has_key?(:country)
        self.country = attributes[:country]
      end
      
      if attributes.has_key?(:country_subdivision)
        self.country_subdivision = attributes[:country_subdivision]
      end
      
      if attributes.has_key?(:line1)
        self.line1 = attributes[:line1]
      end
      
      if attributes.has_key?(:line2)
        self.line2 = attributes[:line2]
      end
      
      if attributes.has_key?(:line3)
        self.line3 = attributes[:line3]
      end
      
      if attributes.has_key?(:postal_code)
        self.postal_code = attributes[:postal_code]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          city == o.city &&
          country == o.country &&
          country_subdivision == o.country_subdivision &&
          line1 == o.line1 &&
          line2 == o.line2 &&
          line3 == o.line3 &&
          postal_code == o.postal_code &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [city, country, country_subdivision, line1, line2, line3, postal_code, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Address" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :city => 'String',
        :country => 'String',
        :country_subdivision => 'String',
        :line1 => 'String',
        :line2 => 'String',
        :line3 => 'String',
        :postal_code => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
