require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
 

module MastercardMerchantOnboarding
  # This class contains the branding details.
  class CheckoutBrand
    include ROXML
    
    xml_name "CheckoutBrand"
    
    # @!attribute checkout_id
    # @return [String] If the checkout brand exists for update action, then the checkout id is mandatory. For example, a4c411by7lob4i81xb6r41i820mw3n151.
    xml_accessor :checkout_id, :from =>"CheckoutId"
   
    # @!attribute checkout_identifier
    # @return [String] Used by the merchant upload response.
    xml_accessor :checkout_identifier, :from =>"CheckoutIdentifier"
   
    # @!attribute name
    # @return [String] Name displayed on Masterpass site during checkout when no logo is provided. For example, XYZ Pizza.
    xml_accessor :name, :from =>"Name"
   
    # @!attribute display_name
    # @return [String] the display name. For example, XYZ Pizza.
    xml_accessor :display_name, :from =>"DisplayName"
   
    # @!attribute production_url
    # @return [String] the production checkout URL. For example, https://xyzpizza.com
    xml_accessor :production_url, :from =>"ProductionUrl"
   
    # @!attribute sandbox_url
    # @return [String] the sandbox checkout URL. For example, https://test.xyzpizza.com
    xml_accessor :sandbox_url, :from =>"SandboxUrl"
   
    # @!attribute logo_url
    # @return [String] Merchant logo displayed on Masterpass site during checkout. If not specified, the Name is displayed. For example, http://xyzpizza.com/logo.jpg
    xml_accessor :logo_url, :from =>"LogoUrl"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :checkout_id => :CheckoutId ,
        :checkout_identifier => :CheckoutIdentifier ,
        :name => :Name ,
        :display_name => :DisplayName ,
        :production_url => :ProductionUrl ,
        :sandbox_url => :SandboxUrl ,
        :logo_url => :LogoUrl ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:checkout_id)
        self.checkout_id = attributes[:checkout_id]
      end
      
      if attributes.has_key?(:checkout_identifier)
        self.checkout_identifier = attributes[:checkout_identifier]
      end
      
      if attributes.has_key?(:name)
        self.name = attributes[:name]
      end
      
      if attributes.has_key?(:display_name)
        self.display_name = attributes[:display_name]
      end
      
      if attributes.has_key?(:production_url)
        self.production_url = attributes[:production_url]
      end
      
      if attributes.has_key?(:sandbox_url)
        self.sandbox_url = attributes[:sandbox_url]
      end
      
      if attributes.has_key?(:logo_url)
        self.logo_url = attributes[:logo_url]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          checkout_id == o.checkout_id &&
          checkout_identifier == o.checkout_identifier &&
          name == o.name &&
          display_name == o.display_name &&
          production_url == o.production_url &&
          sandbox_url == o.sandbox_url &&
          logo_url == o.logo_url &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [checkout_id, checkout_identifier, name, display_name, production_url, sandbox_url, logo_url, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "CheckoutBrand" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :checkout_id => 'String',
        :checkout_identifier => 'String',
        :name => 'String',
        :display_name => 'String',
        :production_url => 'String',
        :sandbox_url => 'String',
        :logo_url => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
