require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/auth_option'
require_relative '../../mastercard_merchant_onboarding/models/checkout_brand'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
require_relative '../../mastercard_merchant_onboarding/models/merchant_acquirer'
require_relative '../../mastercard_merchant_onboarding/models/profile'
require_relative '../../mastercard_merchant_onboarding/models/reward_program'
 

module MastercardMerchantOnboarding
  # This class contains various merchant details.
  class Merchant
    include ROXML
    
    xml_name "Merchant"
    
    # @!attribute sp_merchant_id
    # @return [String] the Service Provider Issued merchant ID. This should be unique per Service Provider. For example, 123456789
    xml_accessor :sp_merchant_id, :from =>"SPMerchantId"
   
    # @!attribute action
    # @return [String] the merchant action. Valid values are C(Create), U(Update) and D(Delete).
    xml_accessor :action, :from =>"Action"
   
    # @!attribute profile
    # @return [Profile] the merchant Profile. Mandatory for Create(Action=C) and Update(Action=U).
    xml_accessor :profile, :from =>"Profile",:as => Profile
   
    # @!attribute reward_program
    # @return [RewardProgram] the reward program information.
    xml_accessor :reward_program, :from =>"RewardProgram",:as => RewardProgram
   
    xml_accessor :checkout_brand, :from =>"CheckoutBrand", :as =>[CheckoutBrand]
   
    # @!attribute auth_option
    # @return [Array<AuthOption>] the authentication options.
    xml_accessor :auth_option, :from =>"AuthOption", :as =>[AuthOption]
   
    # @!attribute merchant_acquirer
    # @return [Array<MerchantAcquirer>] the merchant acquirer.
    xml_accessor :merchant_acquirer, :from =>"MerchantAcquirer", :as =>[MerchantAcquirer]
   
    # @!attribute checkout_level
    # @return [String] the checkout level.
    xml_accessor :checkout_level, :from =>"CheckoutLevel"
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :sp_merchant_id => :SPMerchantId ,
        :action => :Action ,
        :profile => :Profile ,
        :reward_program => :RewardProgram ,
        :checkout_brand => :CheckoutBrand ,
        :auth_option => :AuthOption ,
        :merchant_acquirer => :MerchantAcquirer ,
        :checkout_level => :CheckoutLevel ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:sp_merchant_id)
        self.sp_merchant_id = attributes[:sp_merchant_id]
      end
      
      if attributes.has_key?(:action)
        self.action = attributes[:action]
      end
      
      if attributes.has_key?(:profile)
        self.profile = attributes[:profile]
      end
      
      if attributes.has_key?(:reward_program)
        self.reward_program = attributes[:reward_program]
      end
      
      if attributes.has_key?(:checkout_brand)
        self.checkout_brand = attributes[:checkout_brand]
      end
      
      if attributes.has_key?(:auth_option)
        self.auth_option = attributes[:auth_option]
      end
      
      if attributes.has_key?(:merchant_acquirer)
        self.merchant_acquirer = attributes[:merchant_acquirer]
      end
      
      if attributes.has_key?(:checkout_level)
        self.checkout_level = attributes[:checkout_level]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
    # Overriding setter method checkout_brand
    # @param value 
  	def checkout_brand=(value)
      if (value.is_a?(Array))
        @checkout_brand = value
      else
        @checkout_brand = [] if !@checkout_brand
        @checkout_brand.push value
      end
    end
    # Overriding setter method auth_option
    # @param value 
  	def auth_option=(value)
      if (value.is_a?(Array))
        @auth_option = value
      else
        @auth_option = [] if !@auth_option
        @auth_option.push value
      end
    end
    # Overriding setter method merchant_acquirer
    # @param value 
  	def merchant_acquirer=(value)
      if (value.is_a?(Array))
        @merchant_acquirer = value
      else
        @merchant_acquirer = [] if !@merchant_acquirer
        @merchant_acquirer.push value
      end
    end
	

    # Custom attribute writer method checking allowed values (enum).
    def action=(action)
      allowed_values = ["C", "U", "D"]
      if action && !allowed_values.include?(action)
        fail "invalid value for 'action', must be one of #{allowed_values}"
      end
      @action = action
    end

    # Custom attribute writer method checking allowed values (enum).
    def checkout_level=(checkout_level)
      allowed_values = ["STANDARD", "EXPRESS"]
      if checkout_level && !allowed_values.include?(checkout_level)
        fail "invalid value for 'checkout_level', must be one of #{allowed_values}"
      end
      @checkout_level = checkout_level
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          sp_merchant_id == o.sp_merchant_id &&
          action == o.action &&
          profile == o.profile &&
          reward_program == o.reward_program &&
          checkout_brand == o.checkout_brand &&
          auth_option == o.auth_option &&
          merchant_acquirer == o.merchant_acquirer &&
          checkout_level == o.checkout_level &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [sp_merchant_id, action, profile, reward_program, checkout_brand, auth_option, merchant_acquirer, checkout_level, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Merchant" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :sp_merchant_id => 'String',
        :action => 'String',
        :profile => 'Profile',
        :reward_program => 'RewardProgram',
        :checkout_brand => 'Array<CheckoutBrand>',
        :auth_option => 'Array<AuthOption>',
        :merchant_acquirer => 'Array<MerchantAcquirer>',
        :checkout_level => 'String',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
