require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/acquirer'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
require_relative '../../mastercard_merchant_onboarding/models/merchant_acquirer_brand'
 

module MastercardMerchantOnboarding
  # This class contains the merchant's acquirer details.
  class MerchantAcquirer
    include ROXML
    
    xml_name "MerchantAcquirer"
    
    # @!attribute action
    # @return [String] the Create, Update, Delete action. Used to create (C), update (U) or delete (D) Visa Password.
    xml_accessor :action, :from =>"Action"
   
    # @!attribute acquirer
    # @return [Acquirer] the merchant's acquirer details.
    xml_accessor :acquirer, :from =>"Acquirer",:as => Acquirer
   
    # @!attribute merchant_acquirer_brand
    # @return [MerchantAcquirerBrand] the merchant acquirer brand.
    xml_accessor :merchant_acquirer_brand, :from =>"MerchantAcquirerBrand",:as => MerchantAcquirerBrand
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   
    # @!attribute id
    # @return [String] the acquirer id. Used while merchant upload response.
    xml_accessor :id, :from =>"ID"
   
    # @!attribute name
    # @return [String] the merchant's acquirer name. Used while merchant upload response.
    xml_accessor :name, :from =>"Name"
   
    # @!attribute assigned_merchant_id
    # @return [String] the assigned merchant id. Used while merchant upload response.
    xml_accessor :assigned_merchant_id, :from =>"AssignedMerchantID"
   
    # @!attribute ds_status
    # @return [String] the assigned merchant id. Used while merchant upload response.
    xml_accessor :ds_status, :from =>"DSStatus"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :action => :Action ,
        :acquirer => :Acquirer ,
        :merchant_acquirer_brand => :MerchantAcquirerBrand ,
        :extension_point => :ExtensionPoint ,
        :id => :ID ,
        :name => :Name ,
        :assigned_merchant_id => :AssignedMerchantID ,
        :ds_status => :DSStatus 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:action)
        self.action = attributes[:action]
      end
      
      if attributes.has_key?(:acquirer)
        self.acquirer = attributes[:acquirer]
      end
      
      if attributes.has_key?(:merchant_acquirer_brand)
        self.merchant_acquirer_brand = attributes[:merchant_acquirer_brand]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
      if attributes.has_key?(:id)
        self.id = attributes[:id]
      end
      
      if attributes.has_key?(:name)
        self.name = attributes[:name]
      end
      
      if attributes.has_key?(:assigned_merchant_id)
        self.assigned_merchant_id = attributes[:assigned_merchant_id]
      end
      
      if attributes.has_key?(:ds_status)
        self.ds_status = attributes[:ds_status]
      end
      
    end
    
 
	

    # Custom attribute writer method checking allowed values (enum).
    def action=(action)
      allowed_values = ["C", "U", "D"]
      if action && !allowed_values.include?(action)
        fail "invalid value for 'action', must be one of #{allowed_values}"
      end
      @action = action
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          action == o.action &&
          acquirer == o.acquirer &&
          merchant_acquirer_brand == o.merchant_acquirer_brand &&
          extension_point == o.extension_point &&
          id == o.id &&
          name == o.name &&
          assigned_merchant_id == o.assigned_merchant_id &&
          ds_status == o.ds_status
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [action, acquirer, merchant_acquirer_brand, extension_point, id, name, assigned_merchant_id, ds_status].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "MerchantAcquirer" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :action => 'String',
        :acquirer => 'Acquirer',
        :merchant_acquirer_brand => 'MerchantAcquirerBrand',
        :extension_point => 'ExtensionPoint',
        :id => 'String',
        :name => 'String',
        :assigned_merchant_id => 'String',
        :ds_status => 'String'
        
      }
    end
  end


end
