require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
require_relative '../../mastercard_merchant_onboarding/models/merchant_response_record'
require_relative '../../mastercard_merchant_onboarding/models/summary'
 

module MastercardMerchantOnboarding
  # This class contains the merchant upload response details.
  class MerchantDownload
    include ROXML
    
    xml_name "MerchantDownload"
    
    # @!attribute summary
    # @return [Summary] the summary of the upload request.
    xml_accessor :summary, :from =>"Summary",:as => Summary
   
    # @!attribute merchant_response_record
    # @return [Array<MerchantResponseRecord>] the list of merchants uploaded/edited.
    xml_accessor :merchant_response_record, :from =>"MerchantResponseRecord", :as =>[MerchantResponseRecord]
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   
    # @!attribute version
    # @return [String] the version information.
    xml_accessor :version, :from =>"@Version"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :summary => :Summary ,
        :merchant_response_record => :MerchantResponseRecord ,
        :extension_point => :ExtensionPoint ,
        :version => :Version 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:summary)
        self.summary = attributes[:summary]
      end
      
      if attributes.has_key?(:merchant_response_record)
        self.merchant_response_record = attributes[:merchant_response_record]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
      if attributes.has_key?(:version)
        self.version = attributes[:version]
      end
      
    end
    
 
    # Overriding setter method merchant_response_record
    # @param value 
  	def merchant_response_record=(value)
      if (value.is_a?(Array))
        @merchant_response_record = value
      else
        @merchant_response_record = [] if !@merchant_response_record
        @merchant_response_record.push value
      end
    end
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          summary == o.summary &&
          merchant_response_record == o.merchant_response_record &&
          extension_point == o.extension_point &&
          version == o.version
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [summary, merchant_response_record, extension_point, version].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "MerchantDownload" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :summary => 'Summary',
        :merchant_response_record => 'Array<MerchantResponseRecord>',
        :extension_point => 'ExtensionPoint',
        :version => 'String'
        
      }
    end
  end


end
