require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/checkout_brand'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
require_relative '../../mastercard_merchant_onboarding/models/merchant_acquirer'
 

module MastercardMerchantOnboarding
  # This class contains the merchant response record.
  class MerchantResponseRecord
    include ROXML
    
    xml_name "MerchantResponseRecord"
    
    # @!attribute sp_merchant_id
    # @return [String] the merchant identifier for a specific service provider. For example, 12345.
    xml_accessor :sp_merchant_id, :from =>"SPMerchantId"
   
    # @!attribute error_text
    # @return [String] the error description for failure. For example, Invalid rewards logo URL.
    xml_accessor :error_text, :from =>"ErrorText"
   
    # @!attribute checkout_brand
    # @return [Array<CheckoutBrand>] the checkout branding. Choice (ErrorText or CheckoutBrand).
    xml_accessor :checkout_brand, :from =>"CheckoutBrand", :as =>[CheckoutBrand]
   
    # @!attribute merchant_acquirer
    # @return [Array<MerchantAcquirer>] the merchant acquirer information.
    xml_accessor :merchant_acquirer, :from =>"MerchantAcquirer", :as =>[MerchantAcquirer]
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :sp_merchant_id => :SPMerchantId ,
        :error_text => :ErrorText ,
        :checkout_brand => :CheckoutBrand ,
        :merchant_acquirer => :MerchantAcquirer ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:sp_merchant_id)
        self.sp_merchant_id = attributes[:sp_merchant_id]
      end
      
      if attributes.has_key?(:error_text)
        self.error_text = attributes[:error_text]
      end
      
      if attributes.has_key?(:checkout_brand)
        self.checkout_brand = attributes[:checkout_brand]
      end
      
      if attributes.has_key?(:merchant_acquirer)
        self.merchant_acquirer = attributes[:merchant_acquirer]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
    # Overriding setter method checkout_brand
    # @param value 
  	def checkout_brand=(value)
      if (value.is_a?(Array))
        @checkout_brand = value
      else
        @checkout_brand = [] if !@checkout_brand
        @checkout_brand.push value
      end
    end
    # Overriding setter method merchant_acquirer
    # @param value 
  	def merchant_acquirer=(value)
      if (value.is_a?(Array))
        @merchant_acquirer = value
      else
        @merchant_acquirer = [] if !@merchant_acquirer
        @merchant_acquirer.push value
      end
    end
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          sp_merchant_id == o.sp_merchant_id &&
          error_text == o.error_text &&
          checkout_brand == o.checkout_brand &&
          merchant_acquirer == o.merchant_acquirer &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [sp_merchant_id, error_text, checkout_brand, merchant_acquirer, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "MerchantResponseRecord" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :sp_merchant_id => 'String',
        :error_text => 'String',
        :checkout_brand => 'Array<CheckoutBrand>',
        :merchant_acquirer => 'Array<MerchantAcquirer>',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
