require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/acquirer'
require_relative '../../mastercard_merchant_onboarding/models/address'
require_relative '../../mastercard_merchant_onboarding/models/emails'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
require_relative '../../mastercard_merchant_onboarding/models/phone'
 

module MastercardMerchantOnboarding
  # This class contains various merchant profile details.
  class Profile
    include ROXML
    
    xml_name "Profile"
    
    # @!attribute name
    # @return [String] the merchant name. For example, XYZ Pizza
    xml_accessor :name, :from =>"Name"
   
    # @!attribute doing_bus_as
    # @return [String] the alias - Doing Business As. For example, XYZ Pizza Inc.
    xml_accessor :doing_bus_as, :from =>"DoingBusAs"
   
    # @!attribute fed_tax_id
    # @return [String] the merchant’s Federal Tax ID. For example, 999990001.
    xml_accessor :fed_tax_id, :from =>"FedTaxId"
   
    # @!attribute url
    # @return [String] the merchant’s website URL. For example, www.xyzpizza.com.
    xml_accessor :url, :from =>"Url"
   
    # @!attribute business_category
    # @return [String] the merchant’s business type. For example, Restaurant.
    xml_accessor :business_category, :from =>"BusinessCategory"
   
    # @!attribute address
    # @return [Address] the merchant's address.
    xml_accessor :address, :from =>"Address",:as => Address
   
    # @!attribute phone
    # @return [Phone] the merchant's phone.
    xml_accessor :phone, :from =>"Phone",:as => Phone
   
    # @!attribute emails
    # @return [Emails] the merchant's email addresses.
    xml_accessor :emails, :from =>"Emails",:as => Emails
   
    # @!attribute acquirer
    # @return [Acquirer] the merchant's acquirer.
    xml_accessor :acquirer, :from =>"Acquirer",:as => Acquirer
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :name => :Name ,
        :doing_bus_as => :DoingBusAs ,
        :fed_tax_id => :FedTaxId ,
        :url => :Url ,
        :business_category => :BusinessCategory ,
        :address => :Address ,
        :phone => :Phone ,
        :emails => :Emails ,
        :acquirer => :Acquirer ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:name)
        self.name = attributes[:name]
      end
      
      if attributes.has_key?(:doing_bus_as)
        self.doing_bus_as = attributes[:doing_bus_as]
      end
      
      if attributes.has_key?(:fed_tax_id)
        self.fed_tax_id = attributes[:fed_tax_id]
      end
      
      if attributes.has_key?(:url)
        self.url = attributes[:url]
      end
      
      if attributes.has_key?(:business_category)
        self.business_category = attributes[:business_category]
      end
      
      if attributes.has_key?(:address)
        self.address = attributes[:address]
      end
      
      if attributes.has_key?(:phone)
        self.phone = attributes[:phone]
      end
      
      if attributes.has_key?(:emails)
        self.emails = attributes[:emails]
      end
      
      if attributes.has_key?(:acquirer)
        self.acquirer = attributes[:acquirer]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          name == o.name &&
          doing_bus_as == o.doing_bus_as &&
          fed_tax_id == o.fed_tax_id &&
          url == o.url &&
          business_category == o.business_category &&
          address == o.address &&
          phone == o.phone &&
          emails == o.emails &&
          acquirer == o.acquirer &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [name, doing_bus_as, fed_tax_id, url, business_category, address, phone, emails, acquirer, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Profile" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :name => 'String',
        :doing_bus_as => 'String',
        :fed_tax_id => 'String',
        :url => 'String',
        :business_category => 'String',
        :address => 'Address',
        :phone => 'Phone',
        :emails => 'Emails',
        :acquirer => 'Acquirer',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
