require 'date'
require 'roxml'
require_relative '../../mastercard_merchant_onboarding/models/extension_point'
require_relative '../../mastercard_merchant_onboarding/models/project_key_type'
 

module MastercardMerchantOnboarding
  # This class contains the merchant upload summary.
  class Summary
    include ROXML
    
    xml_name "Summary"
    
    # @!attribute batch_id
    # @return [Integer] the system assigned batch Id. For example, 277800
    xml_accessor :batch_id, :from =>"BatchId"
   
    # @!attribute description
    # @return [String] the description provided during file upload. For example, Merchant file upload.
    xml_accessor :description, :from =>"Description"
   
    # @!attribute upload_timestamp
    # @return [String] the upload time stamp. For example, 2013-05-18T11:45:19.444-05:00
    xml_accessor :upload_timestamp, :from =>"UploadTimestamp"
   
    # @!attribute upload_file_name
    # @return [String] the upload file name. For example, merchant.xml
    xml_accessor :upload_file_name, :from =>"UploadFileName"
   
    # @!attribute success_count
    # @return [Integer] the number of successfully uploaded merchant brandings.
    xml_accessor :success_count, :from =>"SuccessCount"
   
    # @!attribute failure_count
    # @return [Integer] the number of unsuccessfully uploaded merchant brandings.
    xml_accessor :failure_count, :from =>"FailureCount"
   
    # @!attribute error_text
    # @return [String] the description of errors resulting upload failures. For example, Invalid Format.
    xml_accessor :error_text, :from =>"ErrorText"
   
    # @!attribute project_keys
    # @return [ProjectKeyType] the developer environment keys.
    xml_accessor :project_keys, :from =>"ProjectKeys",:as => ProjectKeyType
   
    # @!attribute extension_point
    # @return [ExtensionPoint] the extension point for future enhancement.
    xml_accessor :extension_point, :from =>"ExtensionPoint",:as => ExtensionPoint
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :batch_id => :BatchId ,
        :description => :Description ,
        :upload_timestamp => :UploadTimestamp ,
        :upload_file_name => :UploadFileName ,
        :success_count => :SuccessCount ,
        :failure_count => :FailureCount ,
        :error_text => :ErrorText ,
        :project_keys => :ProjectKeys ,
        :extension_point => :ExtensionPoint 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:batch_id)
        self.batch_id = attributes[:batch_id]
      end
      
      if attributes.has_key?(:description)
        self.description = attributes[:description]
      end
      
      if attributes.has_key?(:upload_timestamp)
        self.upload_timestamp = attributes[:upload_timestamp]
      end
      
      if attributes.has_key?(:upload_file_name)
        self.upload_file_name = attributes[:upload_file_name]
      end
      
      if attributes.has_key?(:success_count)
        self.success_count = attributes[:success_count]
      end
      
      if attributes.has_key?(:failure_count)
        self.failure_count = attributes[:failure_count]
      end
      
      if attributes.has_key?(:error_text)
        self.error_text = attributes[:error_text]
      end
      
      if attributes.has_key?(:project_keys)
        self.project_keys = attributes[:project_keys]
      end
      
      if attributes.has_key?(:extension_point)
        self.extension_point = attributes[:extension_point]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          batch_id == o.batch_id &&
          description == o.description &&
          upload_timestamp == o.upload_timestamp &&
          upload_file_name == o.upload_file_name &&
          success_count == o.success_count &&
          failure_count == o.failure_count &&
          error_text == o.error_text &&
          project_keys == o.project_keys &&
          extension_point == o.extension_point
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [batch_id, description, upload_timestamp, upload_file_name, success_count, failure_count, error_text, project_keys, extension_point].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardMerchantOnboarding.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Summary" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :batch_id => 'Integer',
        :description => 'String',
        :upload_timestamp => 'String',
        :upload_file_name => 'String',
        :success_count => 'Integer',
        :failure_count => 'Integer',
        :error_text => 'String',
        :project_keys => 'ProjectKeyType',
        :extension_point => 'ExtensionPoint'
        
      }
    end
  end


end
